{% if kolla_ansible_ctl_install_type == 'source' %}
# Install Kolla Ansible from source.
{{ kolla_ansible_source_path }}
{% else %}
# Install Kolla Ansible from PyPI.
kolla-ansible=={{ kolla_openstack_release }}
{% endif %}
# Limit the version of ansible used by kolla-ansible to avoid new releases from
# breaking tested code. Changes to this limit should be tested.
ansible<2.7
{% if kolla_ansible_venv_extra_requirements is defined %}
{% for item in kolla_ansible_venv_extra_requirements %}
{{ item }}
{% endfor %}
{% endif %}
